package com.puppy.uhfexample.adapter;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import java.util.List;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.puppy.uhfexample.R;

public class BlueToothInfoAdapter extends RecyclerView.Adapter<BlueToothInfoAdapter.BlueToothInfoHolder> {

    private Context mContext;
    private List<BluetoothDevice> blueToothDeviceInfo;

    public BlueToothInfoAdapter(Context mContext, List<BluetoothDevice> blueToothDeviceInfo) {
        this.mContext = mContext;
        this.blueToothDeviceInfo = blueToothDeviceInfo;
    }

    private View.OnClickListener itemOnClickListener;

    public void setItemOnClickListener(View.OnClickListener itemOnClickListener) {
        this.itemOnClickListener = itemOnClickListener;
    }

    @NonNull
    @Override
    public BlueToothInfoHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(android.R.layout.simple_list_item_1, parent, false);
        if (itemOnClickListener != null) {
            View tv = view.findViewById(android.R.id.text1);
            tv.setBackgroundResource(R.drawable.text_border);
            tv.setFocusable(true);
            tv.requestFocus();
            tv.setOnClickListener(itemOnClickListener);
        }
        return new BlueToothInfoHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull BlueToothInfoHolder holder, int position) {
        BluetoothDevice bluetoothDevice = blueToothDeviceInfo.get(position);
        //    MLog.e("bluetoothDevice = " + bluetoothDevice);
        if (null != bluetoothDevice) {
            String bondState = bluetoothDevice.getBondState() == BluetoothDevice.BOND_BONDED ? "  已绑定" : "";
            String showBtDeviceInfo = bluetoothDevice.getName() + " " + bluetoothDevice.getAddress() + bondState;
            holder.blueToothInfo.setText(showBtDeviceInfo);
            holder.blueToothInfo.setTag(bluetoothDevice);
        }
    }

    @Override
    public int getItemCount() {
        return blueToothDeviceInfo.size();
    }

    static class BlueToothInfoHolder extends RecyclerView.ViewHolder {
        private TextView blueToothInfo;

        BlueToothInfoHolder(@NonNull View itemView) {
            super(itemView);
            blueToothInfo = itemView.findViewById(android.R.id.text1);
            blueToothInfo.setTextSize(14);
        }
    }
}
