package com.puppy.uhfexample.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.PopupMenu;
import android.widget.TextView;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.R;
import com.puppy.uhfexample.bean.TagBankInfo;
import com.puppy.uhfexample.util.MConstant;

import java.util.List;
import java.util.Map;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;


public class DataAdapter extends RecyclerView.Adapter<DataAdapter.MyDataHolder> implements View.OnClickListener {

    private Context mContext;
    private Map<String, TagBankInfo> tagBankInfoMap;
    private List<String> keyList;

    public DataAdapter(Context context, List<String> keyList, Map<String, TagBankInfo> tagBankInfoMap) {
        mContext = context;
        this.keyList = keyList;
        this.tagBankInfoMap = tagBankInfoMap;
    }


    @NonNull
    @Override
    public MyDataHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(R.layout.data_item, viewGroup, false);
        return new MyDataHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull MyDataHolder holder, int position) {
        int mode = MyApp.currentInventoryDataType;
        holder.mTvEpc.setOnClickListener(this);
        holder.mTvTid.setOnClickListener(this);

        holder.mLyTid.setVisibility(mode == 0 ? View.GONE : View.VISIBLE);
//        if (mode == 0) {
//            holder.mLyTid.setVisibility(View.GONE);
//            holder.mLyEpc.setVisibility(View.VISIBLE);
//        } else if (mode == 1) {
//            holder.mLyEpc.setVisibility(View.GONE);
//            holder.mLyTid.setVisibility(View.VISIBLE);
//        } else {
//            holder.mLyTid.setVisibility(View.VISIBLE);
//            holder.mLyEpc.setVisibility(View.VISIBLE);
//        }

        String key = keyList.get(position);
        TagBankInfo mTagBankInfo = tagBankInfoMap.get(key);
        if (mTagBankInfo != null) {
            holder.mTvEpc.setText(mTagBankInfo.getEpcData());
            holder.mTvTid.setText(mTagBankInfo.getTidData());
            holder.mTvSum.setText(mTagBankInfo.getReadNums() + "");
            String snNumber = position + 1 + "";
            holder.mTvSn.setText(snNumber);
        }

    }

    @Override
    public int getItemCount() {
        return keyList.size();
    }

    @Override
    public void onClick(View v) {
        boolean flag = v.getId() == R.id.tv_epc;
        String text = ((TextView) v).getText().toString();
        showPopupWindow((View) v.getParent().getParent(), text, flag);
    }

    public interface CallBack {
        void switchToFragmentAndOperate(String data, boolean ifEPC, int i);
    }

    private CallBack mCallback;

    public void setCallback(CallBack mCallback) {
        this.mCallback = mCallback;
    }

    // 弹框根据点击的选项，将EPC或者TID信息复制并填到对应的界面
    private void showPopupWindow(View v, String data, boolean ifEPC) {
        PopupMenu pMenu = new PopupMenu(mContext, v);
        MenuInflater menuInflater = new MenuInflater(mContext);
        menuInflater.inflate(R.menu.operate_menu, pMenu.getMenu());
        pMenu.setOnMenuItemClickListener(item -> {
            switch (item.getItemId()) {
                case R.id.rwlkTagPage:
                    mCallback.switchToFragmentAndOperate(data, ifEPC, MConstant.RWLK_TAG);
                    break;
                case R.id.searchTagPage:
                    mCallback.switchToFragmentAndOperate(data, ifEPC, MConstant.SEARCH_TAG);
                    break;
            }
            return false;
        });
        pMenu.show();
    }


    static class MyDataHolder extends RecyclerView.ViewHolder {
        private TextView mTvSn;
        private TextView mTvSum;
        private LinearLayout mLyEpc, mLyTid;
        private TextView mTvEpc, mTvTid;

        public MyDataHolder(@NonNull View itemView) {
            super(itemView);
            mLyEpc = itemView.findViewById(R.id.ly_epc);
            mLyTid = itemView.findViewById(R.id.ly_tid);
            mTvSn = itemView.findViewById(R.id.item_tv_sn);
            mTvSum = itemView.findViewById(R.id.item_tv_sum);
            mTvEpc = itemView.findViewById(R.id.tv_epc);
            mTvTid = itemView.findViewById(R.id.tv_tid);
        }

    }


}
