package com.puppy.uhfexample.base;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.util.MConstant;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.RxBus;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.viewbinding.ViewBinding;
import io.reactivex.rxjava3.disposables.Disposable;

public abstract class BaseFragment<P extends BasePresenter, T extends ViewBinding> extends Fragment implements CommonInterface {

    protected P presenter;
    protected MyApp myApp = MyApp.getMyApp();
    protected T binding;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (presenter == null) {
            presenter = getPresenter();
            presenter.attachView((BaseView) this);
        }
        reflectInitLayoutView(container);
        return binding.getRoot();
    }


    private void reflectInitLayoutView(ViewGroup container) {
        if (binding == null) {
            Type mSuperclass = getClass().getGenericSuperclass();
            Class<?> mClass = (Class<?>) ((ParameterizedType) mSuperclass).getActualTypeArguments()[1];
            try {
                Method method = mClass.getDeclaredMethod("inflate", LayoutInflater.class, ViewGroup.class, boolean.class);
                binding = (T) method.invoke(null, getLayoutInflater(), container, false);
            } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                e.printStackTrace();
            }
        }
    }

    protected abstract P getPresenter();


    public void onClick(View v) {

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        init(view);
        subscribeFilterData();
    }

    private void subscribeFilterData() {
        Disposable mDisposable = RxBus.getDefault()
                .autoSubscription(messageEvent -> {
                    switch (messageEvent.whatMsg) {
                        //读写锁杀找标签的地方
                        case MConstant.RWLK_TAG:
                            MLog.e("come to here");
                            FilterData mFData = (FilterData) messageEvent.msgData;
                            String data = mFData.getData();
                            boolean dataBank = mFData.isDataBank();
                            updateFilterData(data, dataBank);
                            break;
                        // 所有UHF功能设置重置，通知UI更新
                        case MConstant.RESET_ALL_CONFIG:
                            refreshUi();
                            break;
                        default:
                            break;
                    }
                });
        RxBus.getDefault().addSubscription(this, mDisposable);
    }

    protected void refreshUi() {
    }

    protected void updateFilterData(String data, boolean dataBank) {

    }

    public void showing() {

    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxBus.getDefault().unSubscribe(this);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (presenter != null)
            presenter.detachView();
        presenter = null;
    }

    public boolean isInventrying() {
        return false;
    }
}
