package com.puppy.uhfexample.presenter;


import android.bluetooth.BluetoothDevice;
import android.content.res.Resources;

import com.cyd.uhf.MConstant;
import com.cyd.uhf.TriggerKeyListener;
import com.puppy.uhf.UHFManager;
import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseView;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.bean.BlueToothTriggerInfo;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.view.fragment.SettingFragment;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.core.ObservableOnSubscribe;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.schedulers.Schedulers;

public class BtTriggerImpl implements BtTriggerContract.BtTriggerPresenter, TriggerKeyListener {

    private BtTriggerContract.BtTriggerView btTriggerView;

    public BtTriggerImpl(BtTriggerContract.BtTriggerView btTriggerView) {
        this.btTriggerView = btTriggerView;
        getUHFManager().addTriggerKeyListener(this);
    }

    @Override
    public void attachView(BaseView baseView) {

    }

    @Override
    public void detachView() {
        getUHFManager().removeTriggerKeyListener(this);
        for (Disposable disposable : allDisposable) {
            disposable.dispose();
        }
    }

    private UHFManager getUHFManager() {
        return MyApp.getMyApp().getUHFManager();
    }

    private List<Disposable> allDisposable = new ArrayList<>();

    @Override
    public void getBTTriggerInfo() {
        Disposable infoDisposable = Observable.create((ObservableOnSubscribe<BlueToothTriggerInfo>) emitter -> {
            BluetoothDevice blDevice = MyApp.getMyApp().getConnectedDevice();
            String btName = null, btMacAddress = null;
            if (blDevice != null) {
                btName = blDevice.getName();
                btMacAddress = blDevice.getAddress();
            }
            int triggerModeVal = getUHFManager().getTriggerMode();
            Resources mRes = MyApp.getMyApp().getResources();
            String triggerMode;
            switch (triggerModeVal) {
                case MConstant.UHF_MODE:
                    triggerMode = mRes.getString(R.string.current_uhf_mode);
                    break;
                case MConstant.EX25_MODE:
                    triggerMode = mRes.getString(R.string.current_ex25_mode);
                    break;
                case MConstant.UNKNOWN_MODE:
                default:
                    triggerMode = mRes.getString(R.string.current_unknown_mode);
                    break;
            }
            String btTriggerSn = getUHFManager().getBlueToothTriggerSN();
            String btHardVer = getUHFManager().getBTHardwareVer();
            String btFirmwareVer = getUHFManager().getBTMBFirmwareVer();
            byte[] inventoryVoiceParam = getUHFManager().getInvVoiceParam();
            boolean isINVVoiceOpen = false;
            int inventoryVoiceLevel = 0;
            if (inventoryVoiceParam != null) {
                isINVVoiceOpen = inventoryVoiceParam[0] == 0x01;
                MLog.e("isInvVoiceOpen = " + inventoryVoiceParam[0]);
                inventoryVoiceLevel = inventoryVoiceParam[1];
                if (inventoryVoiceLevel > 3 || inventoryVoiceLevel < 0) {
                    inventoryVoiceLevel = 0;
                }
            }
            boolean isInvLampOpen = getUHFManager().isInvLampInstructOpen();
            boolean ifFanOpen = getUHFManager().getFanSwitch();
            emitter.onNext(new BlueToothTriggerInfo(btTriggerSn, btName, btMacAddress, triggerMode, btHardVer, btFirmwareVer, inventoryVoiceLevel, isINVVoiceOpen, isInvLampOpen, ifFanOpen));
            getOnceBTBatteryLevel();
        }).subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(blueToothTriggerInfo -> btTriggerView.initInfo(blueToothTriggerInfo));
        allDisposable.add(infoDisposable);

        Disposable mDisposable = Observable.interval(60, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    if (!MyApp.ifUpdateFirmwareOrInventory) {
                        if (MyApp.ifConnectBlue)
                            getOnceBTBatteryLevel();
                        else
                            allDisposable.clear();
                    }
                });
        allDisposable.add(mDisposable);

    }

    private void getOnceBTBatteryLevel() {
        boolean isBtTriggerFragment = MyApp.getMyApp().getCurrentTabFragment().getClass().getSimpleName().equals(SettingFragment.class.getSimpleName());
        MLog.e(" getOnceBTBatteryLevel  isBtTriggerFragment = " + isBtTriggerFragment + " MyApp.getMyApp().getCurrentTabFragment().getClass().getSimpleName() = " + MyApp.getMyApp().getCurrentTabFragment().getClass().getSimpleName());
        if (btTriggerView.isVisibility() && isBtTriggerFragment) {
            String[] battery = getUHFManager().getBTBatteryLevel();
            if (battery != null) {
                Resources res = MyApp.getMyApp().getResources();
                String str = String.format(res.getString(R.string.power_notice),
                        Boolean.parseBoolean(battery[0]) ? res.getString(R.string.charging) : res.getString(R.string.nocharge), battery[1]);
                MLog.e("str = " + str);
                btTriggerView.refreshBatteryLev(str);
            }
        }
    }

    @Override
    public boolean setFanSwitch(boolean isChecked) {
        return getUHFManager().setFanSwitch(isChecked, true);
    }

    @Override
    public boolean setInvLampInstructParam(boolean isChecked) {
        return getUHFManager().setInvLampPara(isChecked, true);
    }

    @Override
    public boolean setInventoryVoiceParam(boolean ifOpenVoice, int voiceLevel) {
        return getUHFManager().setInvVoiceParam(ifOpenVoice, voiceLevel, true);
    }

    @Override
    public void getTriggerKeyAction(int keyAction) {
        btTriggerView.showKeyAction(keyAction);
    }

}
