package com.puppy.uhfexample.presenter;


import com.puppy.uhf.UHFManager;
import com.puppy.uhfexample.base.BaseView;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.bean.UHFParamBean;
import com.puppy.uhfexample.util.MLog;

import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.core.ObservableOnSubscribe;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.schedulers.Schedulers;


public class GeneralSettingsImpl implements GeneralSettingsContract.GeneralSettingsPresenter {

    private GeneralSettingsContract.GeneralSettingsView mGeneralSettingsView;
    private Disposable mDisp;

    public GeneralSettingsImpl(BaseView mBaseView) {
        mGeneralSettingsView = (GeneralSettingsContract.GeneralSettingsView) mBaseView;
    }

    @Override
    public void attachView(BaseView baseView) {

    }

    @Override
    public void detachView() {
        mGeneralSettingsView = null;
        if (mDisp != null) {
            mDisp.dispose();
        }
    }

    private UHFManager getUHFManager() {
        return MyApp.getMyApp().getUHFManager();
    }

    @Override
    public void initConfig() {
        mDisp = Observable.create((ObservableOnSubscribe<UHFParamBean>) emitter -> {
            UHFParamBean mUHFSetting = UHFParamBean.getInstance();
            int power = getUHFManager().powerGet();
            int frequency = getUHFManager().frequencyModeGet();
            mUHFSetting.setPower(power);
            mUHFSetting.setFrequency(frequency);
            MLog.e(" power = " + power + " setFre = " + frequency);
            emitter.onNext(mUHFSetting);
        }).subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(uhfParamBean -> {
                    mGeneralSettingsView.allConfigUpdate(uhfParamBean.getPower(), uhfParamBean.getFrequency());
                });
    }

    @Override
    public boolean sessionModeSet(int var1) {
        return getUHFManager().sessionModeSet(var1,true);
    }

    @Override
    public int sessionModeGet() {
        return getUHFManager().sessionModeGet();
    }

    @Override
    public boolean inventoryWaitTimeSet(int var1, int var2, boolean var3) {
        return getUHFManager().inventoryWaitTimeSet(var1, var2, var3);
    }

    @Override
    public int[] inventoryWaitTimeGet() {
        return getUHFManager().inventoryWaitTimeGet();
    }

    @Override
    public boolean frequenceRangeSet(int[] var1, int var2, boolean var3) {
        return getUHFManager().frequenceRangeSet(var1, var2, var3);
    }

    @Override
    public int[] frequenceRangeGet() {
        return getUHFManager().frequenceRangeGet();
    }

    @Override
    public int frequencyModeGet() {
        return getUHFManager().frequencyModeGet();
    }

    @Override
    public boolean frequencyModeSet(int var1) {
        return getUHFManager().frequencyModeSet(var1);
    }


}
