package com.puppy.uhfexample.presenter;

import android.os.SystemClock;

import com.cyd.uhf.InventoryDataInfo;
import com.cyd.uhf.MConstant;
import com.cyd.uhf.TriggerKeyListener;

import com.puppy.uhf.UHFManager;
import com.puppy.uhfexample.base.BaseView;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.bean.TagBankInfo;
import com.puppy.uhfexample.result.GetRFIDThread;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.ThreadUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class InventoryImpl implements InventoryContract.InventoryPresenter, InventoryDataInfo, TriggerKeyListener {

    private InventoryContract.InventoryView mInventoryView;

    private long startTime, usTim, pauseTime;
    private String tagNumber, readNumber, takeTime;

    public InventoryImpl(BaseView mBaseView) {
        mInventoryView = (InventoryContract.InventoryView) mBaseView;
        getUHFManager().addTriggerKeyListener(this);
    }

    @Override
    public void attachView(BaseView baseView) {

    }

    @Override
    public void detachView() {
        mInventoryView = null;
        getUHFManager().inventoryDataGet(null);
        getUHFManager().removeTriggerKeyListener(this);
    }

/*
    private UHFManager getUHFManager() {
        return MyApp.getMyApp().getUHFManager();
    }
*/

    private UHFManager getUHFManager() {
        return MyApp.getMyApp().getUHFManager();
    }

    @Override
    public void startInventory() {
        MyApp.ifUpdateFirmwareOrInventory = true;
        ifSoundThreadAlive = true;
        playSound();
        getUHFManager().startInventoryTag();
        long tempTime = pauseTime;
        startINVTime = System.currentTimeMillis();
        startTime = startINVTime - tempTime;
        GetRFIDThread.getInstance().setIfPostMsg(true);
        mInventoryView.setViewEnabled(false);
    }

    @Override
    public void stopInventory() {
        ifHaveTag = false;
        ifSoundThreadAlive = false;
        getUHFManager().stopInventory();
        GetRFIDThread.getInstance().setIfPostMsg(false);
        mInventoryView.setViewEnabled(true);
        MyApp.ifUpdateFirmwareOrInventory = false;
    }

    @Override
    public void clearDatas() {
        usTim = pauseTime = 0;
        mInventoryView.clearData(takeTime, tagNumber, readNumber);
    }

    @Override
    public boolean inventoryModeSet(int position, boolean ifsave) {
        return getUHFManager().inventoryModeSet(position, ifsave);
    }

    @Override
    public int inventoryModeGet() {
        return getUHFManager().inventoryModeGet();
    }

    @Override
    public boolean inventoryTagBankSet(int position, boolean ifsave) {
        return getUHFManager().inventoryTagBankSet(position, 0, 0, ifsave);
    }

    @Override
    public int inventoryTagBankGet() {
        int[] tagBankParam = getUHFManager().inventoryTagBankGet();
        if (tagBankParam == null) {
            return 0;
        }
        return tagBankParam[0];
    }

    @Override
    public long getPauseTime() {
        return pauseTime;
    }

    @Override
    public long getUseTime() {
        return usTim;
    }

    @Override
    public long getStartTime() {
        return startTime;
    }

    @Override
    public String getTakeTime() {
        return takeTime;
    }

    @Override
    public String getTagNumber() {
        return tagNumber;
    }

    @Override
    public String getReadNumber() {
        return readNumber;
    }

    @Override
    public void calculatingTime(long endTime) {
        pauseTime = usTim = endTime - startTime; //RFID开始到结束的获取时间
    }

    private Map<String, TagBankInfo> tagBankInfoMap = new HashMap<>();
    private List<String> keyList = new ArrayList<>();

    @Override
    public List<String> getList() {
        return keyList;
    }

    @Override
    public Map<String, TagBankInfo> getTagMap() {
        return tagBankInfoMap;
    }

    private long postDataTime = 0;


    @Override
    public void postResult(String[] tagData) {
        if (tagData == null || tagData.length <= 0) {
            return;
        }
        //++rNumber;
        postDataTime = SystemClock.elapsedRealtime();
        ifHaveTag = true;

        String tid = tagData[0];//获取TID
        String epc = tagData[1]; //拿到EPC
        boolean ifTIDMode = MyApp.currentInventoryDataType > 0;
        String key = ifTIDMode ? tid : epc;
        TagBankInfo tagBankInfo = tagBankInfoMap.get(key);
        if (tagBankInfo == null) {
            tagBankInfo = new TagBankInfo(1, epc, tid);
            keyList.add(key);
        } else {
            int rNums = tagBankInfo.getReadNums() + 1;
            tagBankInfo.setReadNums(rNums);
        }
        tagBankInfoMap.put(key, tagBankInfo);
        mInventoryView.updateUI();
    }

    private boolean ifSoundThreadAlive = true;
    private boolean ifHaveTag = false;
    private long lastTime;

    private void playSound() {
        ThreadUtil.getInstance().getExService().execute(() -> {
            while (MyApp.ifOpenSound && ifSoundThreadAlive) {
                if (ifHaveTag) {
                    //超过1s无数据暂停播放声音
                    if (lastTime != 0 && lastTime - postDataTime > 1000) {
                        ifHaveTag = false;
                        continue;
                    }
                    MyApp.getMyApp().playSound();
                    SystemClock.sleep(600);
                    lastTime = SystemClock.elapsedRealtime();
                }
            }
        });
    }

    @Override
    public void postInventoryRate(long rate) {

    }

    private long startINVTime = 0;
    private long readRate = 0;

    @Override
    public void getInventoryData(String[] data) {
        long cTime = System.currentTimeMillis();//当前时间
        if (startINVTime != 0 && cTime - startINVTime >= 1000) {
            mInventoryView.getInventoryRate(readRate);
            readRate = 0;
            startINVTime = cTime;
        } else if (data != null) {
            ++readRate;
        }
        postResult(data);
    }

    @Override
    public void getTriggerKeyAction(int keyAction) {
        if (MConstant.TRIGGER_KEY_UP == keyAction) {
            MLog.e(" keyAction = "+keyAction+" ifInventoryOrScan = "+MyApp.ifInventoryOrScan);
            if (!MyApp.ifInventoryOrScan) {
                mInventoryView.triggerKeyDown();
            }
        }
    }

    @Override
    public void addInventoryDataGetListener() {
        getUHFManager().inventoryDataGet(this);
    }
}
