package com.puppy.uhfexample.presenter;

import android.os.Environment;
import android.os.SystemClock;


import com.puppy.uhf.UHFManager;
import com.puppy.uhfexample.base.BaseView;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.bean.MessageEvent;
import com.puppy.uhfexample.util.MConstant;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.RxBus;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.core.ObservableOnSubscribe;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.schedulers.Schedulers;

public class SystemSettingsImpl implements SystemSettingsContract.SystemSettingsPresenter {

    private SystemSettingsContract.SystemSettingsView mSystemSettingsView;

    private String currentBinName;
    private List<Disposable> allDisposable = new ArrayList<>();

    public SystemSettingsImpl(BaseView mBaseView) {
        mSystemSettingsView = (SystemSettingsContract.SystemSettingsView) mBaseView;
    }

    @Override
    public void attachView(BaseView baseView) {

    }

    @Override
    public void detachView() {
        mSystemSettingsView = null;
        for (Disposable disposable : allDisposable) {
            disposable.dispose();
        }
    }

    private UHFManager getUHFManager() {
        return MyApp.getMyApp().getUHFManager();
    }


    @Override
    public void initSettingsInfo() {
        Disposable disposable1 = Observable.create((ObservableOnSubscribe<String[]>) emitter -> {
            String firmwareVer = getUHFManager().getFirmwareVer();
            String hardwareVer = getUHFManager().getHardwareVer();
            float temperature = getUHFManager().getModuleTemperature();
            int antennaReturnLoss = getUHFManager().getAntennaReturnLoss();
            emitter.onNext(new String[]{firmwareVer, hardwareVer, String.valueOf(temperature), antennaReturnLoss + ""});
        }).subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread()).subscribe(strings -> mSystemSettingsView.showVersionInfo(strings));

        Disposable timingAcquisitionTemperature = Observable.interval(5000, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    if (!MyApp.ifUpdateFirmwareOrInventory) {
                        float temperature = getUHFManager().getModuleTemperature();
                        int antennaReturnLoss = getUHFManager().getAntennaReturnLoss();
                        mSystemSettingsView.refreshTemperatureAndReturnLoss(String.valueOf(temperature), antennaReturnLoss + "");
                    }
                });
        allDisposable.add(timingAcquisitionTemperature);
        allDisposable.add(disposable1);
    }

    //机器内置sd卡的根目录(具体的目录地址可自行修改拼接处理)
    private String sdcardPath = Environment.getExternalStorageDirectory().getAbsolutePath();


    //sd卡根目录.bin文件的获取
    @Override
    public List<String> getBinFileName() {
        List<String> ar = new ArrayList<>();
        File file = new File(sdcardPath);
        MLog.e("sdcardPath = " + sdcardPath);
        File[] allFiles = file.listFiles();
        MLog.e(" size = " + (allFiles == null));
        if (allFiles != null) {
            for (File allFile : allFiles) {
                String mFileName = allFile.getName();
                MLog.e(" currentFile = " + mFileName);
                if (mFileName.endsWith(".bin")) {
                    ar.add(mFileName);
                }
            }
        }
        return ar;
    }

    @Override
    public void updateFirmware() {
        MyApp.ifUpdateFirmwareOrInventory = true;
        Disposable mDis = Observable.create((ObservableOnSubscribe<Boolean>) emitter -> {
            long startTime = System.currentTimeMillis();
            boolean result = getUHFManager().updateFirmware(sdcardPath + File.separator + currentBinName, currentBinName);
            long endTime = System.currentTimeMillis();
            MLog.e("Firmware address：" + sdcardPath + File.separator + currentBinName + "，Update result：" + result);
            //等待同样的时间，让固件从蓝牙主板写入模块中
            if (result) {
                SystemClock.sleep(endTime - startTime);
            }
            emitter.onNext(result);
        }).subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(result -> {
                    //给予时间等待更新后重新建立连接
                    SystemClock.sleep(1000);
                    MyApp.ifUpdateFirmwareOrInventory = false;
                    mSystemSettingsView.getUpdateResult(result);
                    mSystemSettingsView.goToReConnect();
                });
        allDisposable.add(mDis);
    }

    @Override
    public void updateBTFirmware() {
        MyApp.ifUpdateFirmwareOrInventory = true;
        Disposable mDis = Observable.create((ObservableOnSubscribe<Boolean>) emitter -> {
            boolean result = getUHFManager().updateBTMainBoard(sdcardPath + File.separator + currentBinName, currentBinName);
            MLog.e("Firmware address：" + sdcardPath + File.separator + currentBinName + "，Update result：" + result);
            emitter.onNext(result);
        }).subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(result -> {
                    if (result) {
                        MyApp.ifUpdateFirmwareOrInventory = false;
                        mSystemSettingsView.getUpdateResult(result);
                        mSystemSettingsView.goToReConnect();
                    }
                });
        allDisposable.add(mDis);
    }

    @Override
    public void setBinName(String fileName) {
        currentBinName = fileName;
    }

    @Override
    public void reSetModuleConfig() {
        MyApp.ifUpdateFirmwareOrInventory = true;
        Disposable mDis = Observable.create((ObservableOnSubscribe<Boolean>) emitter -> {
            boolean result = getUHFManager().resetDefaultSetting();
            MLog.e("result = " + result);
            emitter.onNext(result);
            emitter.onComplete();
        }).delay(300, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    MyApp.ifUpdateFirmwareOrInventory = false;
                    if (aBoolean) {
                        MessageEvent event = MessageEvent.obtain();
                        event.whatMsg = MConstant.RESET_ALL_CONFIG;
                        RxBus.getDefault().post(event);
                    }
                    mSystemSettingsView.resetConfigResult(aBoolean);
                    mSystemSettingsView.goToReConnect();
                });
        allDisposable.add(mDis);
    }

}
