package com.puppy.uhfexample.util;

import com.puppy.uhfexample.bean.MessageEvent;

import java.util.HashMap;

import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.disposables.CompositeDisposable;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.functions.Consumer;
import io.reactivex.rxjava3.subjects.BehaviorSubject;
import io.reactivex.rxjava3.subjects.Subject;

public class RxBus {

    private HashMap<String, CompositeDisposable> mSubscriptionMap;
    private static volatile Subject<Object> instance;

    public static RxBus getDefault() {
        return MySingleton.singleton;
    }

    private static class MySingleton {
        final static RxBus singleton = new RxBus();
    }

    private RxBus() {
        instance = BehaviorSubject.create().toSerialized();
    }

    public void post(Object o) {
        instance.onNext(o);
    }

    public <T> Observable toObservable(Class<T> cls) {
        return instance.ofType(cls);
    }

    public Disposable autoSubscription(Consumer<MessageEvent> consumer) {
        return instance.ofType(MessageEvent.class)
                .subscribeOn(AndroidSchedulers.mainThread())
                .subscribe(consumer);
    }

    /**
     * 保存订阅后的subscription
     *
     * @param o
     * @param subscription
     */
    public void addSubscription(Object o, Disposable subscription) {
        if (mSubscriptionMap == null) {
            mSubscriptionMap = new HashMap<>();
        }
        String key = o.getClass().getName();
        if (mSubscriptionMap.get(key) != null) {
            mSubscriptionMap.get(key).add(subscription);
        } else {
            CompositeDisposable compositeSubscription = new CompositeDisposable();
            compositeSubscription.add(subscription);
            mSubscriptionMap.put(key, compositeSubscription);
        }
    }

    /**
     * 取消订阅
     *
     * @param o
     */
    public void unSubscribe(Object o) {
        if (mSubscriptionMap == null) {
            return;
        }

        String key = o.getClass().getName();
        if (!mSubscriptionMap.containsKey(key)) {
            return;
        }
        CompositeDisposable currentDisposable = mSubscriptionMap.get(key);
        if (currentDisposable != null) {
            currentDisposable.dispose();
        }

        mSubscriptionMap.remove(key);
    }

}
