package com.puppy.uhfexample.view;

import android.Manifest;
import android.app.Activity;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;


import com.puppy.uhf.UHFManager;
import com.puppy.uhfexample.R;
import com.puppy.uhfexample.adapter.BlueToothInfoAdapter;
import com.puppy.uhfexample.base.BaseActivity;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.databinding.ActivityBlueBinding;
import com.puppy.uhfexample.util.BlueToothDeviceListener;
import com.puppy.uhfexample.util.BluetoothUtil;
import com.puppy.uhfexample.util.MConstant;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.MUtil;
import com.puppy.uhfexample.util.RequestOpenUtil;
import com.puppy.uhfexample.util.ThreadUtil;
import com.puppy.uhfexample.zxing.CaptureActivity;
import com.puppy.uhfexample.zxing.Constant;
import com.tencent.mmkv.MMKV;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;

public class BlueToothActivity extends BaseActivity {

    private ActivityBlueBinding blueBinding;
    private BluetoothUtil mBlueToothUtil;
    private List<BluetoothDevice> allBlueToothDeviceInfo;
    private BlueToothInfoAdapter mBlueToothInfoAdapter;
    private MMKV mmkv;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        blueBinding = ActivityBlueBinding.inflate(getLayoutInflater());
        setContentView(blueBinding.getRoot());
        mmkv = MMKV.defaultMMKV();
        MyApp.ifInventoryOrScan = false;
        mBlueToothUtil = new BluetoothUtil(this);
        if (!RequestOpenUtil.ifGpsOpen(this)) {
            RequestOpenUtil.openLocationService(this, resultCode -> MUtil.show(resultCode == Activity.RESULT_OK ? R.string.gps_open_success : R.string.gps_open_failed));
        }
        if (!mBlueToothUtil.ifBlueToothEnabled()) {
            RequestOpenUtil.requestOpenBlueTooth(this, resultCode -> {
                boolean flag = resultCode == Activity.RESULT_OK;
                MUtil.show(flag ? R.string.bluetooth_request_opened_successfully : R.string.bluetooth_request_open_failed);
                if (flag) {
                    MLog.e("BlueTooth has been opened,now start to bonded last Device if exist");
                    autoBondedLastDevice();
                } else {
                    blueBinding.scanConnectBluetooth.setEnabled(true);
                    blueBinding.manualConnection.setEnabled(true);
                }
            });
        } else {
            MLog.e("autoBondedLastDevice");
            autoBondedLastDevice();
        }
        RequestOpenUtil.launchAll(this);
        if (getNeedRequestPermission(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.CAMERA).size() > 0) {
            reqPermissions(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.CAMERA);
        }
    }


    private void autoBondedLastDevice() {
        BluetoothDevice lastBondedDevice = mmkv.decodeParcelable(MConstant.LAST_BONDED_DEVICE, BluetoothDevice.class);
        if (lastBondedDevice != null) {
            controlProgressDialog(true);
            blueBinding.scanConnectBluetooth.post(() -> createSocketConnectToDevice(lastBondedDevice));
        } else {
            blueBinding.scanConnectBluetooth.setEnabled(true);
            blueBinding.manualConnection.setEnabled(true);
        }
    }

    private ProgressDialog progressDialog;

    private void controlProgressDialog(boolean ifShow) {
        runOnUiThread(() -> {
            if (progressDialog == null) {
                progressDialog = new ProgressDialog(BlueToothActivity.this);
                progressDialog.setTitle(getString(R.string.Reconnecting));
                progressDialog.setCancelable(false);
            }
            if (!ifShow && progressDialog.isShowing()) {
                progressDialog.cancel();
                progressDialog = null;
            } else if (ifShow && !progressDialog.isShowing()) {
                progressDialog.show();
            }
        });
    }

    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.bluetooth_mode:
            case R.id.manual_connection:
                blueBinding.scanConnectBluetooth.setEnabled(false);
                blueBinding.manualConnection.setEnabled(false);
                blueBinding.uhfModeLayout.setVisibility(View.GONE);
                blueBinding.blueConnectLayout.setVisibility(View.VISIBLE);
                allBlueToothDeviceInfo = new LinkedList<>();
                mBlueToothInfoAdapter = new BlueToothInfoAdapter(this, allBlueToothDeviceInfo);
                mBlueToothInfoAdapter.setItemOnClickListener(v -> {
                    v.setBackgroundResource(R.drawable.text_border_gray);
                    Object ob = v.getTag();
                    if (ob != null) {
                        new Thread(() -> {
                            mBlueToothUtil.stopFindBlueToothDevice();
                            BluetoothDevice device = (BluetoothDevice) ob;
                            MLog.e(" device = " + device.getName());
                            controlProgressDialog(true);
                            createSocketConnectToDevice(device);
                        }).start();
                    }
                });
                blueBinding.showBluetoothDevice.setLayoutManager(new LinearLayoutManager(this));
                blueBinding.showBluetoothDevice.setAdapter(mBlueToothInfoAdapter);
                mBlueToothUtil.findAllBlueToothDevice(new BlueToothDeviceListener() {
                    @Override
                    public void getBondedDevice(Set<BluetoothDevice> devices) {
                        for (BluetoothDevice device : devices) {
                            if (!allBlueToothDeviceInfo.contains(device)) {
                                allBlueToothDeviceInfo.add(device);
                                mBlueToothInfoAdapter.notifyDataSetChanged();
                            }
                        }
                    }

                    @Override
                    public void getNewBlueDevices(BluetoothDevice device) {
                        if (!allBlueToothDeviceInfo.contains(device)) {
                            allBlueToothDeviceInfo.add(device);
                            mBlueToothInfoAdapter.notifyDataSetChanged();
                        }
                    }
                });
                break;
            case R.id.scan_connect_bluetooth:
                mBlueToothUtil.stopFindBlueToothDevice();
                startActivityForResult(new Intent(this, CaptureActivity.class), Constant.RQ_DECODE_QRCODE);
                break;
            case R.id.serialPort_mode:
            default:
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Constant.DECODE_CODE && requestCode == Constant.RQ_DECODE_QRCODE && null != data) {
            String macAddress = data.getStringExtra(Constant.DECODE_TEXT);
            if (macAddress != null) {
                StringBuilder sb = new StringBuilder(macAddress);
                if (!macAddress.contains(":")) {
                    for (int i = 1; i <= 5; i++) {
                        sb.insert(3 * i - 1, ":");
                    }
                }
                BluetoothDevice device = mBlueToothUtil.getBluetoothAdapter().getRemoteDevice(sb.toString());
                controlProgressDialog(true);
                createSocketConnectToDevice(device);
                Log.e("cyd", "decode = " + macAddress);
            }
        }
    }

//    private BluetoothDevice tempDevice;

    private void createSocketConnectToDevice(BluetoothDevice device) {
//        if (tempDevice != null) {
//            return;
//        }
//        tempDevice = device;
        ThreadUtil.getInstance().getExService().execute(() -> {
            mmkv.encode(MConstant.LAST_BONDED_DEVICE, device);
            mBlueToothUtil.createBluetoothSocket(device, (socket, errorCode) -> {
                runOnUiThread(() -> {
                    switch (errorCode) {
                        case MConstant.BLUETOOTH_BONDED_FAILED:
                            MLog.longToast(getString(R.string.bluetooth_binding_failed));
                            break;
                        case MConstant.BLUETOOTH_SOCKET_CONNECT_FAILED:
                            MLog.longToast(getString(R.string.bluetooth_socket_connection_failed));
                            break;
                        case MConstant.BLUETOOTH_SOCKET_CONNECT_SUCCESS:
                            MLog.longToast(getString(R.string.bluetooth_socket_connection_succeeded));
                            if (socket != null) {
                                MyApp.getMyApp().setUHFManager(new UHFManager(socket));
                                MyApp.getMyApp().setConnectedDevice(device);
                                startActivity(new Intent(BlueToothActivity.this, MainActivity.class));
                                finish();
                            }
                            break;
                        default:
                            break;
                    }
                    controlProgressDialog(false);
                    blueBinding.scanConnectBluetooth.setEnabled(true);
                    blueBinding.manualConnection.setEnabled(true);
                });
            });
        });
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        mBlueToothUtil.releaseResource();
    }
}
