package com.puppy.uhfexample.view;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.SystemClock;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;

import com.cyd.uhf.MConstant;
import com.cyd.uhf.TriggerKeyListener;

import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseActivity;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.result.GetRFIDThread;
import com.puppy.uhfexample.result.OnKeyDownListener;
import com.puppy.uhfexample.result.OnKeyListener;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.MUtil;
import com.puppy.uhfexample.util.RequestOpenUtil;
import com.puppy.uhfexample.util.ThreadUtil;
import com.puppy.uhfexample.view.fragment.BtTriggerFragment;
import com.puppy.uhfexample.view.fragment.DestroyFragment;
import com.puppy.uhfexample.view.fragment.GeneralSettingsFragment;
import com.puppy.uhfexample.view.fragment.InventoryFragment;
import com.puppy.uhfexample.view.fragment.LockFragment;
import com.puppy.uhfexample.view.fragment.SearchTagsFragment;
import com.puppy.uhfexample.view.fragment.SettingFragment;
import com.puppy.uhfexample.view.fragment.SystemSettingsFragment;
import com.puppy.uhfexample.view.fragment.TagOperateFragment;
import com.puppy.uhfexample.view.fragment.WriteFragment;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

import androidx.annotation.Nullable;

import static android.os.Build.VERSION.SDK_INT;
import static com.puppy.uhfexample.util.MyFragmentUtil.switchFragment;

/**
 * 创建日期：2021/2/25 18:00
 *
 * @author zhangjq
 * @version 1.0
 * 类说明：
 */

public class MainActivity extends BaseActivity implements RadioGroup.OnCheckedChangeListener, TriggerKeyListener {

    private RadioGroup mRgTabs;
    private FrameLayout mContainerMain;

    private MyApp myApp = MyApp.getMyApp();
    private BaseFragment currentFragment;
    private List<BaseFragment> mainFragmentsList;
    private List<BaseFragment> tagsFragmentsList;
    private List<BaseFragment> settingsFragmentsList;
    private boolean ifRequestPermission = true;
    private boolean firstInit = true;
    //UHF模块标签信息获取线程
    private GetRFIDThread rfidThread = GetRFIDThread.getInstance();
    private RadioButton mTabHome;
    private RadioButton mTabWrite;
    private RadioButton mTabFind;
    private RadioButton mTabSettings;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //请求保持常亮不锁屏
        MyApp.ifConnectBlue = true;
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON, WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.activity_main);
        requestPermission();
//        if (MyApp.getMyApp().getUHFManager() != null)
//            MLog.e("powerOn = " + MyApp.getMyApp().getUHFManager().powerOn());
//          rfidThread.start();
        addBTConnectStatusListener();
        MyApp.getMyApp().getUHFManager().addTriggerKeyListener(this);
        refreshTriggerElectricQuantity();
    }

    private void initView() {
        mRgTabs = findViewById(R.id.tabs_rg);
        mContainerMain = findViewById(R.id.main_container);

        mainFragmentsList = new ArrayList<>();
        mainFragmentsList.add(new InventoryFragment());
        mainFragmentsList.add(new TagOperateFragment());
        mainFragmentsList.add(new SearchTagsFragment());
        mainFragmentsList.add(new SettingFragment());
        myApp.setMainFragmentsList(mainFragmentsList);

        tagsFragmentsList = new ArrayList<>();
        tagsFragmentsList.add(new WriteFragment());
        tagsFragmentsList.add(new LockFragment());
        tagsFragmentsList.add(new DestroyFragment());
        myApp.setTagsFragmentsList(tagsFragmentsList);

        settingsFragmentsList = new ArrayList<>();
        settingsFragmentsList.add(new GeneralSettingsFragment());
        settingsFragmentsList.add(new SystemSettingsFragment());
        settingsFragmentsList.add(new BtTriggerFragment());
        myApp.setSettingsFragmentsList(settingsFragmentsList);

        mRgTabs.setOnCheckedChangeListener(this);
        mTabHome = findViewById(R.id.home_tab);
        mTabWrite = findViewById(R.id.write_tab);
        mTabFind = findViewById(R.id.find_tab);
        mTabSettings = findViewById(R.id.settings_tab);
        int checkedId = mRgTabs.getCheckedRadioButtonId();
        for (int i = 0; i < mRgTabs.getChildCount(); i++) {
            if (mRgTabs.getChildAt(i).getId() == checkedId) {
                currentFragment = switchFragment(currentFragment, this, null, myApp.getMainFragmentsList().get(i), mContainerMain);
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (!ifRequestPermission) {
            recycleResource();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        switchTriggerMode(false);
    }

    @Override
    protected void onStop() {
        super.onStop();
        switchTriggerMode(true);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        removeBTConnectStatusListener();
        ifAlive = false;
        updateBatteryLevel(0, false);
    }

    private void switchTriggerMode(boolean flag) {
        Intent switchIScanKey = new Intent("android.intent.action.UHF_CHECK_TRIGGER");
        switchIScanKey.putExtra("isEnableScan", flag);
        sendBroadcast(switchIScanKey);
    }


    private volatile long lastTime = 0;

    @Override
    public void onBackPressed() {
        long currentTime = SystemClock.currentThreadTimeMillis();
        if (lastTime != 0 && currentTime - lastTime < 500) {
            recycleResource();
        } else {
            MUtil.show(R.string.exit_app);
        }
        lastTime = currentTime;
    }

    private void recycleResource() {
        ifAlive = false;
        rfidThread.destroyThread();
        updateBatteryLevel(0, false);
        switchTriggerMode(true);
        System.exit(0);
    }

    public void ifAllowedChange(boolean isEnabled) {
        mTabHome.setEnabled(isEnabled);
        mTabWrite.setEnabled(isEnabled);
        mTabFind.setEnabled(isEnabled);
        mTabSettings.setEnabled(isEnabled);
    }


    @Override
    public void onCheckedChanged(RadioGroup radioGroup, int checkedId) {
        if (radioGroup != null & currentFragment != null) {
            if (currentFragment.isInventrying()) {
                MUtil.show(R.string.other_operations_are_not_allowed_during_inventory);
            } else {
                for (int i = 0; i < radioGroup.getChildCount(); i++) {
                    if (radioGroup.getChildAt(i).getId() == checkedId) {
                        currentFragment = switchFragment(currentFragment, this, null, myApp.getMainFragmentsList().get(i), mContainerMain);
                        MyApp.getMyApp().setCurrentTabFragment(currentFragment);
                        return;
                    }
                }
            }

        }

    }

    private void requestPermission() {
        //6.0 ~ 10.0 统一的请求
        if (SDK_INT >= Build.VERSION_CODES.M && SDK_INT < Build.VERSION_CODES.R) {
            //检查有无权限
            if (getNeedRequestPermission(Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE).size() > 0) {
                reqPermissions(Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE);
            } else {
                //已有权限走正常流程
                ifRequestPermission = false;
                initView();
            }
        } else if (SDK_INT >= Build.VERSION_CODES.R) {
            RequestOpenUtil.requestOpenFilePermission(this, resultCode -> {
                if (resultCode == Activity.RESULT_OK) {
                    //已有权限走正常流程
                    ifRequestPermission = false;
                    initView();
                }
            });
            RequestOpenUtil.launchAll(this);
        }
    }

    private MyReceiver receiver;

    private void addBTConnectStatusListener() {
        if (receiver == null) {
            receiver = new MyReceiver(this);
            IntentFilter filter = new IntentFilter();
            //蓝牙开关状态广播
            filter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
            //蓝牙断开广播
            filter.addAction(BluetoothDevice.ACTION_ACL_DISCONNECTED);
            registerReceiver(receiver, filter);
        }
    }

    private void removeBTConnectStatusListener() {
        if (receiver != null) {
            unregisterReceiver(receiver);
        }
    }

    @Override
    public void getTriggerKeyAction(int keyAction) {
        runOnUiThread(() -> {
            MLog.e("keyAction = " + keyAction + "  ifInventoryOrScan = " + MyApp.ifInventoryOrScan);
            if (keyAction == MConstant.SWITCH_KEY_UP) {
                MyApp.ifInventoryOrScan = !MyApp.ifInventoryOrScan;
            } else if (MConstant.TRIGGER_KEY_UP == keyAction) {
                if (MyApp.ifInventoryOrScan) {
                    MLog.shortToast(getString(R.string.trigger_mode));
                    MyApp.getMyApp().sendBroadcast(new Intent(MConstant.KEY_BARCODE_STARTSCAN_ACTION));
                }
            }
        });
    }


    static class MyReceiver extends BroadcastReceiver {
        private WeakReference<MainActivity> mWeak;

        public MyReceiver(MainActivity mainActivity) {
            mWeak = new WeakReference<>(mainActivity);
        }

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            MLog.e("bt connect status change " + action);
            if (action != null) {
                switch (action) {
                    case BluetoothAdapter.ACTION_STATE_CHANGED:
                        int blueState = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, 0);
                        switch (blueState) {
                            case BluetoothAdapter.STATE_ON:
                                MUtil.show(R.string.bt_opened);
                                break;
                            case BluetoothAdapter.STATE_OFF:
                                MUtil.show(R.string.bt_closed);
                                mWeak.get().goBlueToothPairPage();
                                break;
                            default:
                                break;
                        }
                        break;
                    case BluetoothDevice.ACTION_ACL_DISCONNECTED:
                        mWeak.get().goBlueToothPairPage();
                        break;
                    default:
                        break;
                }
            }
        }
    }

    private void goBlueToothPairPage() {
        MyApp.ifConnectBlue = false;
        AlertDialog.Builder mDialog = new AlertDialog.Builder(this);
        mDialog.setTitle(R.string.bt_closed_notice);
        mDialog.setCancelable(false);
        AlertDialog dialog = mDialog.create();
        dialog.show();
        new Handler(Looper.getMainLooper()).postDelayed(() -> {
            dialog.cancel();
            ifAlive = false;
            updateBatteryLevel(0, false);
            MLog.e("close icon");
            MyApp.getMyApp().setConnectedDevice(null);
            MyApp.ifInventoryOrScan = false;
            startActivity(new Intent(MainActivity.this, BlueToothActivity.class));
            finish();
        }, 1200);
    }

    /**
     * 监听按键事件（这里主要用于把枪按键），分发到对应fragment处理（一般控制UHF模块线程读取与停止）
     *
     * @param keyCode 当前触发按键值
     * @param event   按键事件
     * @return 是否消费当前按键事件
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (currentFragment instanceof OnKeyDownListener) {
            ((OnKeyDownListener) currentFragment).onKeyDown(keyCode, event);
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (currentFragment instanceof OnKeyListener) {
            ((OnKeyListener) currentFragment).onKeyUp(keyCode, event);
        }
        return super.onKeyUp(keyCode, event);
    }

    //选择跳转写标签或者寻标签页面
    public void operateTag(int pageNo) {
        RadioButton rb = pageNo == 1 ? mTabWrite : mTabFind;
        //防止onCheckedChanged多次调用
        rb.toggle();
    }

    //把枪电量获取线程是否存活
    private boolean ifAlive = true;

    /**
     * 更新把枪电池电量和是否显示电量图标的的广播
     */
    public void refreshTriggerElectricQuantity() {
        ThreadUtil.getInstance().getExService().execute(() -> {
            while (ifAlive) {
                //如果不是首次进入就15分钟获取一次电量
                if (!firstInit) {
                    SystemClock.sleep(15 * 60 * 1000);
                }
                firstInit = false;
                //如果在更新固件或者盘点就不走更新电量
                if (!MyApp.ifUpdateFirmwareOrInventory) {
                    // TODO: 2021/11/22 这里要处理可能在获取电量的时候正好在操作其它的指令而导致电量获取失败的问题
                    String[] battery = MyApp.getMyApp().getUHFManager().getBTBatteryLevel();
                    if (battery != null && battery.length >= 2 && !TextUtils.isEmpty(battery[1])) {
                        int batLev = Integer.parseInt(battery[1]);
                        updateBatteryLevel(batLev, true);
                        MLog.e("show  icon batLev = " + batLev);
                    }
                }
            }
        });
    }

    /**
     * 更新把枪电池状态，并通知到systemUi更新
     *
     * @param batteryLev 当前把枪电量
     * @param ifShowIcon 状态栏是否显示电池图标
     */
    private void updateBatteryLevel(int batteryLev, boolean ifShowIcon) {
        MLog.e("comet to control  icon");
        Intent tEleQun = new Intent("com.trigger.electricquantity");
        // 参数值为当前把枪的电量值
        tEleQun.putExtra("eleQun", batteryLev);
        //是否显示电量图标，false不显示，true显示
        tEleQun.putExtra("ifShowIcon", ifShowIcon);
        sendBroadcast(tEleQun);
    }
}
