package com.puppy.uhfexample.view.fragment;

import android.view.View;
import android.widget.AdapterView;

import com.cyd.uhf.MConstant;

import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.bean.BlueToothTriggerInfo;

import com.puppy.uhfexample.databinding.FragmentBtTriggerBinding;
import com.puppy.uhfexample.presenter.BtTriggerContract;
import com.puppy.uhfexample.presenter.BtTriggerImpl;

import androidx.core.content.ContextCompat;

public class BtTriggerFragment extends BaseFragment<BtTriggerContract.BtTriggerPresenter, FragmentBtTriggerBinding> implements BtTriggerContract.BtTriggerView {


    @Override
    protected BtTriggerImpl getPresenter() {
        return new BtTriggerImpl(this);
    }

    @Override
    public int getLayoutId() {
        return R.layout.fragment_bt_trigger;
    }

    @Override
    public void init(View view) {
        presenter.getBTTriggerInfo();
    }

    private void setInventoryVoice() {
        boolean ifOpenVoice = binding.buzzerSwitch.isChecked();
        int voiceLevel = binding.buzzerVolumeCtl.getSelectedItemPosition();
        presenter.setInventoryVoiceParam(ifOpenVoice, voiceLevel);
    }

    @Override
    public void initInfo(BlueToothTriggerInfo triggerInfo) {
        binding.btConnectStatus.setText(R.string.bt_connected);
        binding.btSn.setText(triggerInfo.getBtTriggerSn());
        binding.btName.setText(triggerInfo.getBtName());
        binding.btMac.setText(triggerInfo.getBtMacAddress());
        binding.triggerMode.setText(triggerInfo.getTriggerMode());
        binding.mainBoardHardwareVer.setText(triggerInfo.getBtHardVer());
        binding.mainBoardSoftVer.setText(triggerInfo.getBtFirmwareVer());
        binding.buzzerSwitch.setChecked(triggerInfo.isINVVoiceOpen());
        binding.buzzerVolumeCtl.setSelection(triggerInfo.getInventoryVoiceLevel());
        binding.indicatorLightSwitch.setChecked(triggerInfo.isInvLampOpen());
        binding.fanSwitch.setChecked(triggerInfo.isIfFanOpen());
        binding.buzzerSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> {
            setInventoryVoice();
        });
        binding.buzzerVolumeCtl.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                setInventoryVoice();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        binding.indicatorLightSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> {
            presenter.setInvLampInstructParam(isChecked);
        });
        binding.fanSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> {
            presenter.setFanSwitch(isChecked);
        });
    }

    @Override
    public boolean isVisibility() {
        return !isHidden();
    }

    @Override
    public void showing() {
        super.showing();
        clearClickBg();
    }


    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (!hidden) {
            clearClickBg();
        }
    }

    private void clearClickBg() {
        binding.tvPOWER.setBackground(null);
        binding.tvGUN.setBackground(null);
        binding.tvFN.setBackground(null);
        binding.tvSW.setBackground(null);
    }

    @Override
    public void refreshBatteryLev(String str) {
        requireActivity().runOnUiThread(() -> binding.triggerBattery.setText(str));
    }

    @Override
    public void showKeyAction(int keyAction) {
        requireActivity().runOnUiThread(() -> {
            switch (keyAction) {
                case MConstant.POWER_KEY_DOWN:
                    //电源键按下
                    binding.tvPOWER.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_blue_light));
                    break;
                case MConstant.POWER_KEY_UP:
                    //电源键开
                    binding.tvPOWER.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_green_dark));
                    break;
                case MConstant.TRIGGER_KEY_DOWN:
                    //巴枪按下
                    binding.tvGUN.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_blue_light));
                    break;
                case MConstant.TRIGGER_KEY_UP:
                    //巴枪松开
                    binding.tvGUN.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_green_dark));
                    break;
                case MConstant.FUNCITON_KEY_DOWN:
                    //功能键按下
                    binding.tvFN.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_blue_light));
                    break;
                case MConstant.FUNCITON_KEY_UP:
                    //功能键松开
                    binding.tvFN.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_green_dark));
                    break;
                case MConstant.SWITCH_KEY_DOWN:
                    //切换键按下
                    binding.tvSW.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_blue_light));
                    break;
                case MConstant.SWITCH_KEY_UP:
                    //切换键松开
                    binding.tvSW.setBackgroundColor(ContextCompat.getColor(requireActivity(), android.R.color.holo_green_dark));
                    break;
                default:
                    break;
            }
        });

    }

    @Override
    public BaseFragment getCurrentFragment() {
        return this;
    }
}
