package com.puppy.uhfexample.view.fragment;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.text.TextUtils;
import android.view.View;
import android.widget.AbsListView;
import android.widget.ArrayAdapter;
import android.widget.CheckedTextView;
import android.widget.GridView;


import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.MyApp;

import com.puppy.uhfexample.databinding.FragmentGeneralSettingsBinding;
import com.puppy.uhfexample.presenter.GeneralSettingsContract;
import com.puppy.uhfexample.presenter.GeneralSettingsImpl;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.MUtil;

import java.util.HashSet;

public class GeneralSettingsFragment extends BaseFragment<GeneralSettingsImpl, FragmentGeneralSettingsBinding> implements GeneralSettingsContract.GeneralSettingsView, View.OnClickListener {

    private AlertDialog ad;
    private HashSet<Integer> hSet = new HashSet();  //存放选中下标
    private int[] getFrqRange = null;  //存放当前区域频率的所有频点
    private int[] cn_840 = {842375, 841125, 844125, 840875, 841875, 841375, 843875, 842625, 843125, 844375, 842125, 843375, 840625, 841625, 843625, 842875};
    private int[] cn_920 = {922375, 921125, 924125, 920875, 921875, 921375, 923875, 922625, 923125, 924375, 922125, 923375, 920625, 921625, 923625, 922875};
    private int[] eu = {867500, 866300, 865700, 866900};
    private int[] us = {915750, 927250, 902750, 915250, 903250, 926750, 910750, 922750, 906750, 926250, 904250, 920250, 919250, 909250, 918750, 917750,
            905250, 904750, 925250, 921750, 914750, 913750, 922250, 911250, 911750, 903750, 908750, 905750, 912250, 906250, 917250, 914250, 907250, 918250,
            916250, 910250, 907750, 924750, 909750, 919750, 916750, 913250, 923750, 908250, 925750, 912750, 924250, 921250, 920750, 923250};


    @Override
    public int getLayoutId() {
        return R.layout.fragment_general_settings;
    }

    @Override
    public void init(View view) {
        binding.setPower.setOnClickListener(this);
        binding.getPower.setOnClickListener(this);
        binding.setFrequency.setOnClickListener(this);
        binding.getFrequency.setOnClickListener(this);
        binding.getSession.setOnClickListener(this);
        binding.setSession.setOnClickListener(this);
        binding.btGetWaitTime.setOnClickListener(this);
        binding.btSetWaitTime.setOnClickListener(this);
        binding.getFrequencyRang.setOnClickListener(this);
        binding.setFrequencyRang.setOnClickListener(this);
        presenter.initConfig();
        binding.inventorySound.setOnCheckedChangeListener((compoundButton, isChecked) -> MyApp.ifOpenSound = isChecked);
    }


    @Override
    protected GeneralSettingsImpl getPresenter() {
        return new GeneralSettingsImpl(this);
    }


    @Override
    public void refreshPower(int power) {
        binding.spTransmitPower.setSelection(power);
    }

    @Override
    public void refreshFrequency(int frequency) {
        binding.spRegionalFrequency.setSelection(frequency);
    }

    @Override
    public void allConfigUpdate(int power, int frequency) {
        binding.spTransmitPower.setSelection(power - 5);
        binding.spRegionalFrequency.setSelection(frequency);
    }

    @Override
    protected void refreshUi() {
        binding.getPower.performClick();
        binding.getFrequency.performClick();
        binding.showWorkFrequency.setText(null);
        binding.getSession.performClick();
        binding.btGetWaitTime.performClick();
    }

    @Override
    public void onClick(View v) {
        boolean ifOperateSuccess = false;
        switch (v.getId()) {
            case R.id.setPower:
                int pow = Integer.parseInt(binding.spTransmitPower.getSelectedItem() + "");
                ifOperateSuccess = MyApp.getMyApp().getUHFManager().powerSet(pow);
                break;
            case R.id.getPower:
                int getpows = MyApp.getMyApp().getUHFManager().powerGet();
                binding.spTransmitPower.setSelection(getpows - 5);
                ifOperateSuccess = getpows != -1;
                break;
            case R.id.setFrequency:
                int frequency = Integer.parseInt(binding.spRegionalFrequency.getSelectedItemPosition() + "");
                ifOperateSuccess = presenter.frequencyModeSet(frequency);
                break;
            case R.id.getFrequency:
                int getFreq = presenter.frequencyModeGet();
                MLog.e("getFreq = " + getFreq);
                ifOperateSuccess = getFreq >= 0 && getFreq <= 3;
                if (ifOperateSuccess) {
                    binding.spRegionalFrequency.setSelection(getFreq);
                }
                break;
            case R.id.get_session:
                int sessionMode = presenter.sessionModeGet();
                ifOperateSuccess = sessionMode > -1 && sessionMode < 4;
                if (ifOperateSuccess) {
                    binding.spSettingSession.setSelection(sessionMode);
                }
                break;
            case R.id.set_session:
                int vals = binding.spSettingSession.getSelectedItemPosition();
                ifOperateSuccess = presenter.sessionModeSet(vals);
                break;
            case R.id.bt_get_wait_time:
                int[] times = presenter.inventoryWaitTimeGet();
                if (times == null) {
                    ifOperateSuccess = false;
                } else {
                    ifOperateSuccess = true;
                    binding.etInventoryTime.setText(times[0] + "");
                    binding.etSleepTime.setText(times[1] + "");
                }
                break;
            case R.id.bt_set_wait_time:
                String waitTime = binding.etInventoryTime.getText().toString();
                String sleepTime = binding.etSleepTime.getText().toString();
                boolean isChecked = binding.cbPowerOffAndSaved.isChecked();
                if (TextUtils.isEmpty(waitTime) || TextUtils.isEmpty(sleepTime)) {
                    MUtil.show(R.string.time_empty_notice);
                    return;
                } else {
                    ifOperateSuccess = presenter.inventoryWaitTimeSet(Integer.parseInt(waitTime), Integer.parseInt(sleepTime), isChecked);
                }
                break;
            case R.id.get_frequency_rang:
                binding.showWorkFrequency.setText(null);
                int[] getFrqRange = null; //当前实际设置的频点
                getFrqRange = presenter.frequenceRangeGet();
                if (getFrqRange == null) {
                    MUtil.show(R.string.failed);
                    return;
                }
                for (int i = 0; i < getFrqRange.length; i++) {
                    if (i < getFrqRange.length - 1)
                        binding.showWorkFrequency.append(getFrqRange[i] + ",");
                    else
                        binding.showWorkFrequency.append(getFrqRange[i] + "");
                }
                ifOperateSuccess = true;
                break;
            default:
                return;
            case R.id.set_frequency_rang:
                selectFrq();
                return;
        }
        if (!isHidden())
            MUtil.show(ifOperateSuccess ? R.string.success : R.string.failed);
    }

    //设置需要的频点
    private void selectFrq() {
        getFrqRange = null;
        int area_fq = presenter.frequencyModeGet();
        switch (area_fq) {
            case 0:
                getFrqRange = cn_840;
                break;
            case 1:
                getFrqRange = cn_920;
                break;
            case 2:
                getFrqRange = eu;
                break;
            case 3:
                getFrqRange = us;
                break;
        }
        if (getFrqRange != null) {
            hSet.clear();
            ad = new AlertDialog.Builder(getContext()).create();
            ad.setTitle(R.string.selectable_frequency_range);
            GridView gd = new GridView(getContext());
            gd.setChoiceMode(AbsListView.CHOICE_MODE_MULTIPLE);
            gd.setNumColumns(3);
            Integer getFrqRanges[] = new Integer[getFrqRange.length];
            for (int i = 0; i < getFrqRange.length; i++) {
                getFrqRanges[i] = getFrqRange[i];
            }
            ArrayAdapter adapter = new ArrayAdapter(getContext(), android.R.layout.simple_list_item_single_choice, getFrqRanges);
            gd.setOnItemClickListener((parent, view, position, id) -> {
                CheckedTextView cb = view.findViewById(android.R.id.text1);
                if (cb.isChecked())
                    hSet.add(position);
                else
                    hSet.remove(position);
            });
            gd.setAdapter(adapter);
            ad.setView(gd);
            ad.setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.sure), (dialog, which) -> {
                int[] frqSel = new int[hSet.size()]; //实际选中的频点
                int i = 0;
                for (Integer integer : hSet) {
                    frqSel[i] = getFrqRange[integer];
                    i++;
                }
                boolean ifSuccess = presenter.frequenceRangeSet(frqSel, frqSel.length, false);
                MUtil.show(ifSuccess ? R.string.success : R.string.failed);
            });
            ad.setButton(DialogInterface.BUTTON_NEGATIVE, getString(R.string.cancel), (dialog, which) -> ad.dismiss());

            ad.create();
            ad.show();
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        presenter.detachView();
    }

}
