package com.puppy.uhfexample.view.fragment;

import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.RadioGroup;


import com.puppy.uhfexample.R;
import com.puppy.uhfexample.adapter.DataAdapter;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.base.FilterData;
import com.puppy.uhfexample.MyApp;
import com.puppy.uhfexample.bean.MessageEvent;

import com.puppy.uhfexample.databinding.FragmentInventoryBinding;
import com.puppy.uhfexample.presenter.InventoryContract;
import com.puppy.uhfexample.presenter.InventoryImpl;
import com.puppy.uhfexample.result.BackResult;
import com.puppy.uhfexample.result.GetRFIDThread;
import com.puppy.uhfexample.util.MConstant;
import com.puppy.uhfexample.util.MLog;
import com.puppy.uhfexample.util.MUtil;
import com.puppy.uhfexample.util.RxBus;
import com.puppy.uhfexample.view.MainActivity;

import androidx.recyclerview.widget.LinearLayoutManager;


public class InventoryFragment extends BaseFragment<InventoryContract.InventoryPresenter, FragmentInventoryBinding> implements InventoryContract.InventoryView,
        RadioGroup.OnCheckedChangeListener, View.OnClickListener, BackResult {

    private DataAdapter mDataAdapter;


    @Override
    public int getLayoutId() {
        return R.layout.fragment_inventory;
    }

    private void initView() {
        binding.tvClear.setOnClickListener(this);
        binding.rgInventory.setOnCheckedChangeListener(this);
        binding.btGetInventoryMode.setOnClickListener(this);
        binding.btSetInventoryMode.setOnClickListener(this);
        binding.spType.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int position, long id) {
                presenter.clearDatas();
                MyApp.currentInventoryDataType = position;
                //     mDataAdapter.setMode(position);
                position = Math.min(position, 1);
                boolean result = presenter.inventoryTagBankSet(position, false);
                MLog.e("inventoryTagBankSet：" + result);
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });
//        int result = presenter.inventoryTagBankGet();
        MLog.e("inventoryTagBankGet： result = " /*+ result*/);
    }

    @Override
    public void init(View view) {
        initView();
        binding.rvData.setLayoutManager(new LinearLayoutManager(getContext()));
        mDataAdapter = new DataAdapter(getContext(), presenter.getList(), presenter.getTagMap());
        binding.rvData.setAdapter(mDataAdapter);
        mDataAdapter.setCallback((data, ifEPC, pageNo) -> {
            if (getActivity() != null && getActivity() instanceof MainActivity) {
                ((MainActivity) getActivity()).operateTag(pageNo);
                MessageEvent msg = MessageEvent.obtain();
                msg.whatMsg = MConstant.RWLK_TAG;
                msg.msgData = new FilterData(ifEPC, data);
                RxBus.getDefault().post(msg);
            }
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        GetRFIDThread.getInstance().setBackResult(this);
        GetRFIDThread.getInstance().setSearchTag(false);
        presenter.addInventoryDataGetListener();
    }

    @Override
    protected void refreshUi() {
        super.refreshUi();
        binding.btGetInventoryMode.performClick();
        binding.spType.setSelection(0);
    }

    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (!hidden) {
            GetRFIDThread.getInstance().setBackResult(this);
            presenter.addInventoryDataGetListener();
        }
    }

    @Override
    protected InventoryImpl getPresenter() {
        return new InventoryImpl(this);
    }

    @Override
    public void onCheckedChanged(RadioGroup radioGroup, int checkedId) {
        switch (checkedId) {
            case R.id.rb_start:
                presenter.startInventory();
                if (getActivity() != null && getActivity() instanceof MainActivity) {
                    ((MainActivity) getActivity()).ifAllowedChange(false);
                }
                break;
            case R.id.rb_stop:
                presenter.stopInventory();
                if (getActivity() != null && getActivity() instanceof MainActivity) {
                    ((MainActivity) getActivity()).ifAllowedChange(true);
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.tv_clear:
                if (GetRFIDThread.getInstance().isIfPostMsg()) {
                    MUtil.show(R.string.notice_clean_data);
                } else {
                    presenter.clearDatas();
                }
                break;
            case R.id.bt_get_inventory_mode:
                int value = presenter.inventoryModeGet();
                boolean getInvResult = value >= 0 && value < 5;
                MLog.e("value=" + value);
                if (getInvResult) {
                    binding.spMode.setSelection(value);
                }
                setResult(getInvResult);
                break;
            case R.id.bt_set_inventory_mode:
                boolean flag = presenter.inventoryModeSet(binding.spMode.getSelectedItemPosition(),
                        binding.cbSaveInventoryMode.isChecked());
                setResult(flag);
                break;
            default:
                break;
        }
    }

    @Override
    public boolean isInventrying() {
        return GetRFIDThread.getInstance().isIfPostMsg();
    }

    @Override
    public void setSpModeEnabled(boolean isEnabled) {
//        binding.spMode.setEnabled(isEnabled);
    }

    @Override
    public void setClearEnabled(boolean isEnabled) {
        binding.tvClear.setEnabled(isEnabled);
    }

    @Override
    public void setViewEnabled(boolean isEnabled) {
        binding.spMode.setEnabled(isEnabled);
        binding.tvClear.setEnabled(isEnabled);
        binding.cbSaveInventoryMode.setEnabled(isEnabled);
        binding.btGetInventoryMode.setEnabled(isEnabled);
        binding.btSetInventoryMode.setEnabled(isEnabled);
        binding.spType.setEnabled(isEnabled);
    }

    @Override
    public void clearData(String takeTime, String tagNumber, String readNumber) {
/*        dataMap.clear();
        realDataMap.clear();
        realKeyList.clear();
        tidList.clear();*/
        presenter.getTagMap().clear();
        presenter.getList().clear();
        binding.tvTime.setText("");
        binding.tvTotal.setText("");
        binding.tvVelocity.setText("");
        mDataAdapter.notifyDataSetChanged();
    }

    @Override
    public void updateUI() {
        requireActivity().runOnUiThread(() -> {
            long endTime = System.currentTimeMillis();
            presenter.calculatingTime(endTime);
            binding.tvTime.setText(/*presenter.getTakeTime() + */presenter.getUseTime() + "");//花费的时间
            binding.tvTotal.setText(/*presenter.getTagNumber() + */presenter.getTagMap().size() + "");//标签数量
            mDataAdapter.notifyDataSetChanged();
   /*         if (MyApp.ifOpenSound) {
                MyApp.getMyApp().playSound();
            }*/
        });
    }

    @Override
    public void postResult(String[] tagData) {
        presenter.postResult(tagData);
    }

    @Override
    public void postInventoryRate(final long rate) {
        requireActivity().runOnUiThread(() -> {
            binding.tvVelocity.setText("" + rate /*+ presenter.getReadNumber()*/); // 张数/秒
        });
    }

    @Override
    public void getInventoryRate(long readRate) {
        requireActivity().runOnUiThread(() -> {
            binding.tvVelocity.setText("" + readRate /*+ presenter.getReadNumber()*/); // 张数/秒
        });
    }

    @Override
    public void onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_F8 || keyCode == KeyEvent.KEYCODE_BUTTON_4 || keyCode == KeyEvent.KEYCODE_F4) {
            boolean flag = !GetRFIDThread.getInstance().isIfPostMsg();
            if (flag) {
                presenter.startInventory();
            } else {
                presenter.stopInventory();
            }
        }
    }

    private void setResult(boolean flag) {
        if (flag) {
            MUtil.show(R.string.success);
        } else {
            MUtil.show(R.string.failed);
        }
    }

    @Override
    public void triggerKeyDown() {
        if (!isHidden()) {
            requireActivity().runOnUiThread(() -> {
                if (binding.rbStart.isChecked()) {
                    binding.rbStop.toggle();
                } else {
                    binding.rbStart.toggle();
                }
            });
        }
    }
}
