package com.puppy.uhfexample.view.fragment;

import android.annotation.SuppressLint;
import android.view.KeyEvent;
import android.view.View;


import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.MyApp;

import com.puppy.uhfexample.databinding.FragmentSearchTagBinding;
import com.puppy.uhfexample.presenter.SearchContract;
import com.puppy.uhfexample.presenter.SearchImpl;
import com.puppy.uhfexample.result.BackResult;
import com.puppy.uhfexample.result.GetRFIDThread;
import com.puppy.uhfexample.util.MUtil;
import com.puppy.uhfexample.view.MainActivity;

import static android.text.TextUtils.isEmpty;

public class SearchTagsFragment extends BaseFragment<SearchContract.SearchPresenter, FragmentSearchTagBinding> implements SearchContract.SearchView,
        View.OnClickListener, BackResult {

    private String content;
    private short maxValue = -29, minValue = -70; //RSSI的最大值和最小值

    @Override
    public int getLayoutId() {
        return R.layout.fragment_search_tag;
    }

    @Override
    public void init(View view) {
        binding.etFindTagsData.setText("1234567890ABCDEF12345678");
        binding.btSearchClearFilter.setOnClickListener(this);
        binding.btSearchFilter.setOnClickListener(this);
        binding.btStartFind.setOnClickListener(this);
        content = getText(R.string.current_Tag).toString();
    }

    @Override
    public void updateFilterData(String data, boolean dataBank) {
        super.updateFilterData(data, dataBank);
        binding.etFindTagsData.setText(data);
        binding.spFindTagsStorage.setSelection(dataBank ? 0 : 1);
        binding.etFindTagsAddress.setText(dataBank ? "32" : "0");
        binding.etFindTagsLength.setText("96");
    }

    @Override
    protected SearchContract.SearchPresenter getPresenter() {
        return new SearchImpl(this);
    }

    @Override
    public void onResume() {
        super.onResume();
        GetRFIDThread.getInstance().setBackResult(this);
        GetRFIDThread.getInstance().setSearchTag(true);
        presenter.addInventoryDataGetListener();
    }

/*    @Override
    public void onPause() {
        MLog.e("查找 onPause");
        GetRFIDThread.getInstance().setSearchTag(false);
        super.onPause();
    }*/

    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (hidden) {
            GetRFIDThread.getInstance().setSearchTag(false);
        } else {
            GetRFIDThread.getInstance().setBackResult(this);
            GetRFIDThread.getInstance().setSearchTag(true);
            presenter.addInventoryDataGetListener();
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.bt_search_filter:
                filter();
                break;
            case R.id.bt_search_clear_filter:
                clearFilter();
                break;
            case R.id.bt_start_find:
                startOrStopFind();
                break;
        }
    }

    private void filter() {
        if (ifNotNull()) {
            int address = Integer.parseInt(binding.etFindTagsAddress.getText().toString());
            int length = Integer.parseInt(binding.etFindTagsLength.getText().toString());
            int value = binding.spFindTagsStorage.getSelectedItemPosition();
            String data = binding.etFindTagsData.getText().toString();
            boolean ifSaved = binding.cbFindTagsSaved.isChecked();
            boolean status = presenter.setFilterData(MyApp.UHF[value], address, length, data, ifSaved);
            //定频915MHZ,定功率25
            if (status && presenter.frequency91520MHZRangSet(ifSaved)) {
                MUtil.show(R.string.fiter_success);
            } else {
                MUtil.show(R.string.fiter_failed);
            }
        } else {
            MUtil.show(R.string.data_cannot_be_null);
        }
    }

    private void clearFilter() {
        int value = binding.spFindTagsStorage.getSelectedItemPosition();
        String data = binding.etFindTagsData.getText().toString();
        boolean ifSaved = binding.cbFindTagsSaved.isChecked();
        //清除过滤，重置区域为美标
        boolean status = presenter.clearFilterData(MyApp.UHF[value], data, ifSaved);
        MUtil.show(status ? R.string.clean_success : R.string.clean_failed);
    }

    private void startOrStopFind() {
        boolean flag = !GetRFIDThread.getInstance().isIfPostMsg();
        if (flag) {
            presenter.startInventoryTag();
            if (getActivity() != null && getActivity() instanceof MainActivity) {
                ((MainActivity) getActivity()).ifAllowedChange(false);
            }
            binding.btSearchClearFilter.setEnabled(false);
            binding.btSearchFilter.setEnabled(false);
            binding.spFindTagsStorage.setEnabled(false);
        } else {
            if (getActivity() != null && getActivity() instanceof MainActivity) {
                ((MainActivity) getActivity()).ifAllowedChange(true);
            }
            binding.btSearchClearFilter.setEnabled(true);
            binding.btSearchFilter.setEnabled(true);
            binding.spFindTagsStorage.setEnabled(true);
            presenter.stopInventory();
        }
        GetRFIDThread.getInstance().setIfPostMsg(flag);
        binding.btStartFind.setText(flag ? R.string.stop_find : R.string.start_find);
    }

    private boolean ifNotNull() {
        return !isEmpty(binding.etFindTagsAddress.getText()) && !isEmpty(binding.etFindTagsLength.getText()) && !isEmpty(binding.etFindTagsData.getText());
    }

    @Override
    public void getInvData(String[] data) {
        postResult(data);
    }

    @Override
    public void postResult(String[] tagData) {
        if (tagData != null) {
            String epc = tagData[1];
            final String rssiStr = tagData[2]; //为负数的字符串

            int floatBit = Integer.parseInt(rssiStr.substring(rssiStr.length() - 1, rssiStr.length())); // 小数位的值
            int intgerBit = Integer.parseInt(rssiStr.substring(0, rssiStr.length() - 2)); // 整数位的值
            if (floatBit > 5) { //四舍五入操作
                --intgerBit;
            } else {
                ++intgerBit;
            }
            int rssi = intgerBit;
            // int length = maxValue - minValue; //取差值
            if (rssi >= maxValue) {
                rssi = maxValue;
            } else if (rssi <= minValue) {
                rssi = minValue;
            }
            rssi -= (minValue - 1);
            final int finalRssi = rssi;
            final int finalRealRssi = intgerBit;

            final String epcStr = content + epc;
            getActivity().runOnUiThread(new Runnable() {
                @SuppressLint("SetTextI18n")
                @Override
                public void run() {
                    binding.tvCurrentTag.setText(epcStr);
                    binding.pbSingnalStrength.setProgress(finalRssi);
                    binding.tvRssi.setText(getString(R.string.current_rssi) + finalRealRssi);
                    presenter.playSound(finalRssi);
                }
            });
        } else { //无数据时候清空UI显示
            getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    binding.tvCurrentTag.setText(null);
                    binding.tvRssi.setText(null);
                    binding.pbSingnalStrength.setProgress(0);
                }
            });
        }
    }

    @Override
    public void postInventoryRate(long rate) {

    }

    @Override
    public void onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_F8) //把枪按钮被按下,默认值为138
            startOrStopFind();
    }

    @Override
    public void triggerKeyDown() {
        if (!isHidden()) {
            requireActivity().runOnUiThread(this::startOrStopFind);
        }
    }
}
