package com.puppy.uhfexample.view.fragment;

import android.content.Intent;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


import com.puppy.uhfexample.BuildConfig;
import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseFragment;
import com.puppy.uhfexample.MyApp;

import com.puppy.uhfexample.databinding.FragmentSystemSettingsBinding;
import com.puppy.uhfexample.presenter.SystemSettingsContract;
import com.puppy.uhfexample.presenter.SystemSettingsImpl;
import com.puppy.uhfexample.view.BlueToothActivity;

import java.util.List;

import androidx.annotation.StringRes;
import androidx.appcompat.app.AlertDialog;


public class SystemSettingsFragment extends BaseFragment<SystemSettingsContract.SystemSettingsPresenter, FragmentSystemSettingsBinding> implements SystemSettingsContract.SystemSettingsView, View.OnClickListener {

    private AlertDialog firUpdateDialog;
    private AlertDialog loadingDialog;
    private String currentBinName;

    @Override
    public int getLayoutId() {
        return R.layout.fragment_system_settings;
    }

    @Override
    public void init(View view) {
        binding.firmwareUpdate.setOnClickListener(this);
        binding.btFirmwareUpdate.setOnClickListener(this);
        binding.btSelectBin.setOnClickListener(this);
        binding.btReset.setOnClickListener(this);
        binding.tvSoftwareVersion.setText(BuildConfig.VERSION_NAME);
        presenter.initSettingsInfo();
    }


    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.bt_reset:
                showLoadingDialog(R.string.reset_setting_wait);
                presenter.reSetModuleConfig();
                break;
            case R.id.firmware_update:
                showLoadingDialog(R.string.firmware_update_wait);
                presenter.updateFirmware();
                break;
            case R.id.bt_select_bin:
                showSelectDialog();
                break;
            case R.id.bt_firmware_update:
                showLoadingDialog(R.string.firmware_update_wait);
                presenter.updateBTFirmware();
                break;
            default:
                break;
        }
    }

    @Override
    protected SystemSettingsImpl getPresenter() {
        return new SystemSettingsImpl(this);
    }


    @Override
    public void showVersionInfo(String[] args) {
        binding.tvFirmwareVersion.setText(args[0]);
        binding.tvHardwareVersion.setText(args[1]);
        binding.tvTemperature.setText(args[2]);
        binding.tvAntennaReturnLoss.setText(args[3]);
    }

    @Override
    public void getUpdateResult(boolean result) {
        cancelDialog(false);
        if (result) {
            binding.firmwareUpdate.setEnabled(false);
            binding.btFirmwareUpdate.setEnabled(false);
            Toast.makeText(myApp, R.string.firmware_update_success, Toast.LENGTH_SHORT).show();
        } else {
            Toast.makeText(myApp, R.string.firmware_update_failed, Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void resetConfigResult(boolean flag) {
        cancelDialog(false);
    }

    @Override
    public void refreshTemperatureAndReturnLoss(String temperature, String antennaReturnLoss) {
        binding.tvTemperature.setText(temperature);
        binding.tvAntennaReturnLoss.setText(antennaReturnLoss);
    }

    private void showSelectDialog() {
        if (firUpdateDialog == null) {
            firUpdateDialog = new AlertDialog.Builder(requireActivity()).create();
            firUpdateDialog.setTitle(getString(R.string.select_firemaware_ver));
            List<String> allBinFile = presenter.getBinFileName();
            ListView showBin = new ListView(getActivity());
            showBin.setOnItemClickListener((parent, view, position, id) -> {
                currentBinName = ((TextView) view.findViewById(android.R.id.text1)).getText().toString();
                binding.selectedFirmwareName.setText(requireActivity().getString(R.string.firmware_name) + currentBinName);
                presenter.setBinName(currentBinName);
                cancelDialog(true);
                if (currentBinName.startsWith("UM")) {
                    binding.firmwareUpdate.setEnabled(true);
                    binding.btFirmwareUpdate.setEnabled(false);
                } else if (currentBinName.startsWith("UR")) {
                    binding.firmwareUpdate.setEnabled(false);
                    binding.btFirmwareUpdate.setEnabled(true);
                }
            });
            showBin.setAdapter(new ArrayAdapter<>(requireContext(), android.R.layout.simple_list_item_1, allBinFile));
            firUpdateDialog.setView(showBin);
            firUpdateDialog.show();
        } else {
            firUpdateDialog.show();
        }
    }

    private TextView showContent;

    private void showLoadingDialog(@StringRes int contentId) {
        if (loadingDialog == null) {
            loadingDialog = new AlertDialog.Builder(requireActivity()).create();
            View view = View.inflate(getActivity(), R.layout.loading_progress, null);
            showContent = view.findViewById(R.id.loading_msg);
            showContent.setText(contentId);
            loadingDialog.setView(view);
            loadingDialog.setCancelable(false);
            loadingDialog.show();
        } else {
            showContent.setText(contentId);
            loadingDialog.show();
        }
    }

    private void cancelDialog(boolean flag) {
        if (flag) {
            if (firUpdateDialog != null) {
                firUpdateDialog.cancel();
                firUpdateDialog = null;
            }
        } else {
            if (loadingDialog != null) {
                loadingDialog.cancel();
                loadingDialog = null;
                showContent = null;
            }
        }

    }

    @Override
    public void goToReConnect() {
        new Handler(Looper.getMainLooper()).postDelayed(() -> {
            MyApp.getMyApp().setConnectedDevice(null);
            startActivity(new Intent(requireActivity(), BlueToothActivity.class));
            requireActivity().finish();
        }, 1500);
    }

    @Override
    public void onDestroyView() {
        presenter.detachView();
        super.onDestroyView();
    }
}
