package com.puppy.uhfexample.view.fragment;

import android.text.TextUtils;
import android.view.View;


import com.puppy.uhfexample.R;
import com.puppy.uhfexample.base.BaseFragment;

import com.puppy.uhfexample.databinding.FragmentWriteBinding;
import com.puppy.uhfexample.presenter.WriteContract;
import com.puppy.uhfexample.presenter.WriteImpl;
import com.puppy.uhfexample.util.MLog;

import static android.text.TextUtils.isEmpty;


public class WriteFragment extends BaseFragment<WriteImpl, FragmentWriteBinding> implements WriteContract.WriteView, View.OnClickListener {
    private boolean ifChecked;

    @Override
    public int getLayoutId() {
        return R.layout.fragment_write;
    }

    @Override
    public void init(View view) {
        binding.etFilterAddress.setText("32");
        binding.etFilterLength.setText("96");
        binding.etFilterData.setText("1234567890ABCDEF12345678");
        binding.spFilterStorage.setSelection(0);
        binding.btReadTag.setOnClickListener(this);
        binding.btWriteTag.setOnClickListener(this);
        binding.etAccessPassword.setText("00000000");
        binding.etReadData.setText("1234567890ABCDEF12345678");

        binding.spReadStorage.setSelection(1);
    }

    @Override
    public void updateFilterData(String data, boolean dataBank) {
        super.updateFilterData(data, dataBank);
        binding.etFilterData.setText(data);
        binding.spFilterStorage.setSelection(dataBank ? 0 : 1);
        binding.etFilterAddress.setText(dataBank ? "32" : "0");
        binding.etFilterLength.setText("96");
    }


    @Override
    protected WriteImpl getPresenter() {
        return new WriteImpl(this);
    }


    @Override
    public void onClick(View v) {
        if (!ifNotNull()) {
            MLog.shortToast(getString(R.string.data_cannot_be_null));
        }
        ifChecked = binding.cbOpenFilter.isChecked();
        boolean status = false;
        String result;
        String pwd = binding.etAccessPassword.getText().toString();
        int filterArea = binding.spFilterStorage.getSelectedItemPosition() + 1;
        int filterAddress = Integer.parseInt(binding.etFilterAddress.getText().toString());
        int filterLength = Integer.parseInt(binding.etFilterLength.getText().toString());
        String filterData = binding.etFilterData.getText().toString();
        int readArea = binding.spReadStorage.getSelectedItemPosition();
        int readAddress = Integer.parseInt(binding.etReadAddress.getText().toString());
        int readLength = Integer.parseInt(binding.etReadLength.getText().toString());
        String readData = binding.etReadData.getText().toString();
        switch (v.getId()) {
            case R.id.bt_read_tag:
                if (ifChecked) {
                    result = presenter.readTag(pwd, filterArea, filterAddress, filterLength, filterData, readArea, readAddress, readLength);
                    MLog.e("result11 = " + result + " pwd = " + pwd + " farea = " + filterArea + " filterLength = " + filterLength
                            + " filterData = " + filterData + " readArea = " + readArea + " readAddress = " + readAddress + " readLength = " + readLength);
                } else {
                    result = presenter.readTag(pwd, 0, 0, 0, "00000000", readArea, readAddress, readLength);
                    MLog.e("result222 = " + result + " pwd = " + pwd + " readArea = " + readArea + " readAddress = " + readAddress + " readLength = " + readLength);
                }
                if (TextUtils.isEmpty(result)) {
                    MLog.shortToast(getString(R.string.read_failed));
                } else {
                    MLog.shortToast(getString(R.string.read_success));
                    binding.etReadData.setText(result);
                }
                break;
            case R.id.bt_write_tag:
                if (ifChecked) {
                    status = presenter.writeTag(pwd, filterArea, filterAddress, filterLength, filterData, readArea, readAddress, readLength, readData);
                } else {
                    if (1 == readArea) {
                        MLog.e("参数：\npwd=" + pwd + "\nAddress=" + readAddress + "\nLength=" + readLength + "\nreadData=" + readData);
                        status = presenter.writeDataToEpc(pwd, readAddress, readLength, readData);
                        MLog.e("写入结果：" + status + " 区域：" + binding.spReadStorage.getSelectedItem());
                    } else {
                        status = presenter.writeTag(pwd, 0, 0, 0, "0", readArea, readAddress, readLength, readData);
                    }
//                    status = presenter.writeTag(pwd, 0, 0, 0, "0", readArea, readAddress, readLength, readData);
                    MLog.e("写入结果：" + status);
                }
                if (status) {
                    MLog.shortToast(getString(R.string.write_success));
                } else {
                    MLog.shortToast(getString(R.string.write_failed));
                }
                break;
            default:
                break;
        }
    }

    private boolean ifNotNull() {
        return !isEmpty(binding.etFilterAddress.getText()) && !isEmpty(binding.etFilterLength.getText()) && !isEmpty(binding.etFilterData.getText()) &&
                !isEmpty(binding.etReadAddress.getText()) && !isEmpty(binding.etReadLength.getText()) && !isEmpty(binding.etReadData.getText()) &&
                !isEmpty(binding.etAccessPassword.getText());
    }

    @Override
    public void onDestroyView() {
        presenter.detachView();
        super.onDestroyView();
    }
}
